#ifndef GESTUREWIDGET_H
#define GESTUREWIDGET_H

#include <QPushButton>
#include <QProxyStyle>

class QLabel;
class QString;
class TextLabel;
class QHBoxLayout;
class QVBoxLayout;

class GestureWidget : public QPushButton
{
    Q_OBJECT
public:
    explicit GestureWidget(QWidget *parent = nullptr);
    ~GestureWidget() override = default;

    void setGestureDescription(const QString &text);
    void setGestureAction(const QString &text);

protected:
    void focusOutEvent(QFocusEvent *e) override;

public slots:
    void themeChanged();
    void onToggled(bool checked);

signals:
    void getFocus();
    void loseFocus();

private:
    TextLabel* m_gestureDescription = nullptr;
    TextLabel* m_gestureAction = nullptr;
    QLabel *m_playIcon = nullptr;

    QHBoxLayout* m_gestureDescriptionLayout = nullptr;
    QVBoxLayout* m_mainLayout = nullptr;

    void updateActionMaxWidth();
    void initProperty();
    void initConnections();
    QLabel* createPlayIconLabel();
    QHBoxLayout* createGestureNameLayout();
    QVBoxLayout* createMainLayout();
    QString placeholderTextColor() const;
    QString placeholderTextCheckedColor() const;
    void setActionTextDefaultColor();
    void setActionTextCheckedColor();
};

class IconLabelStyle : public QProxyStyle
{
    Q_OBJECT
public:
    static IconLabelStyle* getStyle();

    void drawControl(ControlElement element, const QStyleOption *option, QPainter *painter, const QWidget *widget) const;

private:
    IconLabelStyle(QStyle *style = nullptr) : QProxyStyle(style) {}
};

#endif // GESTUREWIDGET_H
