#include "touchpadsettingsui.h"
#include <QDBusInterface>
#include <QDebug>
#include <QGuiApplication>

#include <unistd.h>
#include <ukuisdk/kylin-com4cxx.h>
#include <ukcc/widgets/uslider.h>
#include <kslider.h>


const quint32 maxCycleNum = 3;
const QByteArray themeSchemas = "org.ukui.style";
const QString iconThemeName = "icon-theme-name";

#define KYLIN_USER_GUIDE_PATH              "/"
#define KYLIN_USER_GUIDE_SERVICE           "com.kylinUserGuide.hotel"
#define KYLIN_USER_GUIDE_INTERFACE         "com.guide.hotel"
#define WAYLAND_ENV(a) (a==true ? 0:1)

static const std::string PROJECT_INTEL  = "V10SP1-edu";

TouchpadUI::TouchpadUI(QWidget *parent)
    : QWidget(parent)
    , m_cycleNum(0)
    , m_iconThemeGSettings(nullptr)
{
    waylandEnvFlag = QGuiApplication::platformName().startsWith(QLatin1String("wayland"));
    mVlayout = new QVBoxLayout(this);
    mVlayout->setContentsMargins(0, 0, 0, 0);
    mVlayout->setMargin(0);
    if (KDKGetPrjCodeName() == PROJECT_INTEL) {
        projectCodeName = "intelGif/";
    } else {
        projectCodeName = "";
    }
    initUI();
    initConnection();
    monitorIconThemeChange();

}

TouchpadUI::~TouchpadUI()
{
    if(m_iconThemeGSettings) {
        m_iconThemeGSettings->deleteLater();
        m_iconThemeGSettings = nullptr;
    }
}

QFrame* TouchpadUI::myLine()
{
    QFrame *line = new QFrame(this);
    line->setMinimumSize(QSize(0, 1));
    line->setMaximumSize(QSize(16777215, 1));
    line->setLineWidth(0);
    line->setFrameShape(QFrame::HLine);
    line->setFrameShadow(QFrame::Sunken);

    return line;
}

void TouchpadUI::initUI()
{
    //触控板 文案控件
    mTouchpadSetTitleLabel = new QLabel(this);
    mTouchpadSetTitleLabel->setText(tr("Touchpad"));
    QHBoxLayout *h_touchpad_layout = new QHBoxLayout(this);
    h_touchpad_layout->setContentsMargins(16, 0, 0, 0);
    h_touchpad_layout->addWidget(mTouchpadSetTitleLabel);
    mVlayout->addLayout(h_touchpad_layout);

    QFrame *touchpadFrame = new QFrame(this);
    touchpadFrame->setMaximumSize(16777215, 16777215);
    touchpadFrame->setFrameShape(QFrame::Shape::Box);
    QVBoxLayout *touchpadLyt = new QVBoxLayout(touchpadFrame);
    touchpadLyt->setMargin(0);

    /* MouseDisable */
    mMouseDisableWidget = new QWidget(this);
    mMouseDisableWidget->setMaximumSize(16777215, 60);

    QHBoxLayout *MouseDisableHLayout = new QHBoxLayout(mMouseDisableWidget);
    mMouseDisableBtn = new KSwitchButton(this);
    mMouseDisableLabel = new QLabel(tr("Disable touchpad when using the mouse"), this);
    MouseDisableHLayout->addSpacing(6);
    MouseDisableHLayout->addWidget(mMouseDisableLabel);
    MouseDisableHLayout->addStretch();
    MouseDisableHLayout->addWidget(mMouseDisableBtn);
    MouseDisableHLayout->addSpacing(6);
    mMouseDisableWidget->setLayout(MouseDisableHLayout);
    //add to layout
    touchpadLyt->addSpacing(6);
    touchpadLyt->addWidget(mMouseDisableWidget);
    QFrame *mouseAndSpeedLine = myLine();
    touchpadLyt->addWidget(mouseAndSpeedLine);


    /* CursorSpeed */
    mPointerSpeedWidget = new QWidget(this);
    mPointerSpeedWidget->setMaximumSize(16777215, 60);

    QHBoxLayout *pointerSpeedHLayout = new QHBoxLayout(mPointerSpeedWidget);

    mPointerSpeedLabel = new QLabel(tr("Pointer Speed"), this);
    mPointerSpeedSlowLabel = new QLabel(tr("Slow"), this);
    mPointerSpeedFastLabel = new QLabel(tr("Fast"), this);
    mPointerSpeedSlider = new KSlider(Qt::Horizontal);
    mPointerSpeedSlider->setMinimum(11);
    mPointerSpeedSlider->setMaximum(80);
    mPointerSpeedSlider->setTracking(true);
    pointerSpeedHLayout->addSpacing(6);
    pointerSpeedHLayout->addWidget(mPointerSpeedLabel);
    pointerSpeedHLayout->addSpacing(144);
    pointerSpeedHLayout->addWidget(mPointerSpeedSlowLabel);
    pointerSpeedHLayout->addWidget(mPointerSpeedSlider);
    pointerSpeedHLayout->addWidget(mPointerSpeedFastLabel);
    pointerSpeedHLayout->addSpacing(6);
    mPointerSpeedWidget->setLayout(pointerSpeedHLayout);
    //add to layout
    touchpadLyt->addWidget(mPointerSpeedWidget);
    QFrame *speedAndTypingLine = myLine();
    touchpadLyt->addWidget(speedAndTypingLine);

    /* TypingDisableFrame */
    mTypingDisableWidget = new QWidget(this);
    mTypingDisableWidget->setMaximumSize(16777215,60);

    QHBoxLayout *TypingDisableHLayout = new QHBoxLayout(mTypingDisableWidget);

    mTypingDisableBtn = new KSwitchButton(this);
    mTypingDisableLabel = new QLabel(tr("Disable touchpad when typing"), this);
    TypingDisableHLayout->addSpacing(6);
    TypingDisableHLayout->addWidget(mTypingDisableLabel);
    TypingDisableHLayout->addStretch();
    TypingDisableHLayout->addWidget(mTypingDisableBtn);
    TypingDisableHLayout->addSpacing(6);
    mTypingDisableWidget->setLayout(TypingDisableHLayout);
    //add to layout
    touchpadLyt->addWidget(mTypingDisableWidget);
    QFrame *typingAndClickLine = myLine();
    touchpadLyt->addWidget(typingAndClickLine);

    /* click */
    mClickWidget = new QWidget(this);
    mClickWidget->setMaximumSize(16777215, 50);

    QHBoxLayout *ClickHLayout = new QHBoxLayout(mClickWidget);

    mClickBtn = new KSwitchButton(this);
    mClickLabel = new QLabel(tr("Touch and click on the touchpad"), this);
    ClickHLayout->addSpacing(6);
    ClickHLayout->addWidget(mClickLabel);
    ClickHLayout->addStretch();
    ClickHLayout->addWidget(mClickBtn);
    ClickHLayout->addSpacing(6);
    mClickWidget->setLayout(ClickHLayout);
    //add to layout
    touchpadLyt->addWidget(mClickWidget);
    QFrame *clickAndSlideLine = myLine();
    touchpadLyt->addWidget(clickAndSlideLine);

    /* ScrollSlideFrame */
    mScrollSlideWidget = new QWidget(this);
    mScrollSlideWidget->setMaximumSize(16777215, 60);

    QHBoxLayout *ScrollSlideHLayout = new QHBoxLayout(mScrollSlideWidget);

    mScrollSlideBtn = new KSwitchButton(this);
    mScrollSlideLabel = new QLabel(tr("Scroll bar slides with finger"), this);
    ScrollSlideHLayout->addSpacing(6);
    ScrollSlideHLayout->addWidget(mScrollSlideLabel);
    ScrollSlideHLayout->addStretch();
    ScrollSlideHLayout->addWidget(mScrollSlideBtn);
    ScrollSlideHLayout->addSpacing(6);
    mScrollSlideWidget->setLayout(ScrollSlideHLayout);
    //add to layout
    touchpadLyt->addWidget(mScrollSlideWidget);
    QFrame *slideAndAreaLine = myLine();
    touchpadLyt->addWidget(slideAndAreaLine);

    /* ScrollAreaFrame */
    mScrollAreaWidget = new QWidget(this);
    mScrollAreaWidget->setMaximumSize(16777215, 60);
    QHBoxLayout *ScrollAreaHLayout = new QHBoxLayout(mScrollAreaWidget);

    mScrollAreaLabel = new QLabel(tr("Scrolling area"), this);
    mScrollTypeComBox = new QComboBox(this);
    mScrollTypeComBox->addItem(tr("Two-finger scrolling in the middle area"), V_FINGER_KEY);
    mScrollTypeComBox->addItem(tr("Edge scrolling"), V_EDGE_KEY);
    mScrollTypeComBox->addItem(tr("Disable scrolling"), N_SCROLLING);
    ScrollAreaHLayout->addSpacing(6);
    ScrollAreaHLayout->addWidget(mScrollAreaLabel);
    ScrollAreaHLayout->addSpacing(144);
    ScrollAreaHLayout->addWidget(mScrollTypeComBox);
    ScrollAreaHLayout->addSpacing(6);
    mScrollAreaWidget->setLayout(ScrollAreaHLayout);

    //add to layout
    touchpadLyt->addWidget(mScrollAreaWidget);


    mVlayout->addWidget(touchpadFrame);
    mVlayout->addStretch();

#if WAYLAND_ENV(waylandEnvFlag)
    // "手势" Label
     QLabel *gestureLabel = new QLabel(tr("gesture"));
     QVBoxLayout *vLabelLayout = new QVBoxLayout;
     vLabelLayout->setContentsMargins(16, 0, 0, 0);
     vLabelLayout->addWidget(gestureLabel);
     mVlayout->addSpacing(42);
     mVlayout->addLayout(vLabelLayout);

     /* TouchpadGestureFrame */

     QFrame *touchpadGestureFrame = new QFrame(this);
     touchpadGestureFrame->setMaximumSize(16777215, 16777215);
     touchpadGestureFrame->setFrameShape(QFrame::Shape::Box);
     QVBoxLayout *vPadLayout = new QVBoxLayout(touchpadGestureFrame);
     vPadLayout->setContentsMargins(0, 0, 0, 0);

     // "手势显示" Frame
    QFrame *gestureFrame = new QFrame(touchpadGestureFrame);
    gestureFrame->setMaximumSize(16777215, 16777215);
    gestureFrame->setFrameShadow(QFrame::Plain);
    gestureFrame->setFrameShape(QFrame::Shape::Box);
    QHBoxLayout *gestureLayout = new QHBoxLayout(gestureFrame);
    gestureLayout->setContentsMargins(0, 0, 0, 0);
    gestureLayout->setMargin(16);
    // Item 选择部分
    QWidget *gestureItem = new QWidget(gestureFrame);
    QVBoxLayout *vLayout_1 = new QVBoxLayout(gestureItem);
    vLayout_1->setContentsMargins(0, 0, 0, 0);
    vLayout_1->setMargin(0);
    vLayout_1->setSpacing(4);

    //threeFingerClick
    GestureWidget *threeFingerClick = new GestureWidget(gestureItem);
    threeFingerClick->setGestureDescription(tr("three fingers click"));
    threeFingerClick->setGestureAction(tr("show global search"));
    vLayout_1->addWidget(threeFingerClick);

    //fourSwipeDown
    GestureWidget *fourSwipeDown = new GestureWidget(gestureItem);
    fourSwipeDown->setGestureDescription(tr("swipe three fingers down"));
    fourSwipeDown->setGestureAction(tr("show desktop"));
    vLayout_1->addWidget(fourSwipeDown);

    //fourSwipeUp
    GestureWidget *fourSwipeUp = new GestureWidget(gestureItem);
    fourSwipeUp->setGestureDescription(tr("swipe three fingers up"));
    fourSwipeUp->setGestureAction(tr("open the multitasking view"));
    vLayout_1->addWidget(fourSwipeUp);

    //horizenThreeSwipe
    GestureWidget *horizenThreeSwipe = new GestureWidget(gestureItem);
    horizenThreeSwipe->setGestureDescription(tr("swipe three fingers horizontally"));
    horizenThreeSwipe->setGestureAction(tr("switch windows"));
    vLayout_1->addWidget(horizenThreeSwipe);

    //fourFingerClick
    GestureWidget *fourFingerClick = new GestureWidget(gestureItem);
    fourFingerClick->setGestureDescription(tr("four fingers click"));
    fourFingerClick->setGestureAction(tr("show sidebar"));
    vLayout_1->addWidget(fourFingerClick);

    //horizenFourSwipe
    GestureWidget *horizenFourSwipe = NULL;
    if (KDKGetPrjCodeName() != PROJECT_INTEL) {
      horizenFourSwipe = new GestureWidget(gestureItem);
      horizenFourSwipe->setGestureDescription(tr("swipe four fingers horizontally"));
      horizenFourSwipe->setGestureAction(tr("switch desktop"));
      vLayout_1->addWidget(horizenFourSwipe);
    }
    gestureLayout->addWidget(gestureItem);
    gestureLayout->addSpacing(24);

    // Gif 播放部分
    QWidget *videoWidget = new QWidget(gestureFrame);
    videoWidget->setFixedSize(356, 402);
    QLabel *video_label = new QLabel(videoWidget);
    QVBoxLayout *videoLayout = new QVBoxLayout(videoWidget);
    videoLayout->setContentsMargins(0, 0, 0, 0);
//    videoLayout->setMargin(16);
    videoLayout->addWidget(video_label);
    m_movie = new QMovie(gestureFrame);
    m_movie->setFileName(":/gif/resources/"+projectCodeName+"三指轻点.gif");
    m_movie->setScaledSize(QSize(356, 402));
    m_movie->start();
    m_movie->stop();
    video_label->setMovie(m_movie);
    gestureLayout->addWidget(videoWidget);

    vPadLayout->addWidget(gestureFrame);
    touchpadGestureFrame->adjustSize();

    mVlayout->addWidget(touchpadGestureFrame);


     // "更多手势" Button
    QPushButton *moreGestureBtn = new QPushButton(tr("more gesture"));
    QHBoxLayout *vMoreGestureButtonLayout = new QHBoxLayout;
    vMoreGestureButtonLayout->setContentsMargins(0, 0, 0, 0);
    vMoreGestureButtonLayout->addWidget(moreGestureBtn);
    vMoreGestureButtonLayout->addStretch();
    mVlayout->addSpacing(6);
    mVlayout->addLayout(vMoreGestureButtonLayout);
    mVlayout->addStretch();

    m_gestureArray.append(threeFingerClick);
    m_gestureArray.append(fourSwipeDown);
    m_gestureArray.append(fourSwipeUp);
    m_gestureArray.append(horizenThreeSwipe);
    m_gestureArray.append(fourFingerClick);
    if (KDKGetPrjCodeName() != PROJECT_INTEL) {
        m_gestureArray.append(horizenFourSwipe);
    }
    connect(threeFingerClick, &GestureWidget::clicked, this, [=]() {
        playGif(ScreenGestureType::threeFingerClick);
    });
    connect(fourSwipeDown, &GestureWidget::clicked, this, [=]() {
        playGif(ScreenGestureType::threeFingerSwipeDown);
    });
    connect(fourSwipeUp, &GestureWidget::clicked, this, [=]() {
        playGif(ScreenGestureType::threeFingerSwipeUp);
    });
    connect(horizenThreeSwipe, &GestureWidget::clicked, this, [=]() {
        playGif(ScreenGestureType::threeFingersHorizontalSwipe);
    });
    connect(fourFingerClick, &GestureWidget::clicked, this, [=]() {
        playGif(ScreenGestureType::fourFingerClick);
    });
    if (KDKGetPrjCodeName() != PROJECT_INTEL) {
        connect(horizenFourSwipe, &GestureWidget::clicked, this, [=]() {
            playGif(ScreenGestureType::fourFingersHorizontalSwipe);
        });
    }

    connect(moreGestureBtn, &KBorderButton::clicked, this, [=]() {
        QString pluName = "ukui-control-center/Touchpad";
        qDebug() << pluName;
        QString service_name = "com.kylinUserGuide.hotel_" + QString::number(getuid());
        QDBusInterface interface(service_name,
                                 KYLIN_USER_GUIDE_PATH,
                                 KYLIN_USER_GUIDE_INTERFACE,
                                 QDBusConnection::sessionBus(),
                                 this);
        QDBusMessage msg = interface.call("showGuide", pluName);
    });

    foreach (auto gesture, m_gestureArray) {
        connect(gesture, &GestureWidget::loseFocus, this, &TouchpadUI::stopGif);
        connect(this, &TouchpadUI::iconThemeChangedSignal, gesture, &GestureWidget::themeChanged);
    }
    connect(threeFingerClick, &GestureWidget::loseFocus, this, &TouchpadUI::stopGif);
    connect(fourSwipeDown, &GestureWidget::loseFocus, this, &TouchpadUI::stopGif);
    connect(fourSwipeUp, &GestureWidget::loseFocus, this, &TouchpadUI::stopGif);
    connect(horizenThreeSwipe, &GestureWidget::loseFocus, this, &TouchpadUI::stopGif);
    connect(fourFingerClick, &GestureWidget::loseFocus, this, &TouchpadUI::stopGif);
    if (KDKGetPrjCodeName() != PROJECT_INTEL) {
        connect(horizenFourSwipe, &GestureWidget::loseFocus, this, &TouchpadUI::stopGif);
    }
#endif
}

void TouchpadUI::initConnection()
{
    QByteArray touchpadId(kTouchpadSchemas);

    if (QGSettings::isSchemaInstalled(touchpadId)) {
        mTouchpadGsetting = new QGSettings(kTouchpadSchemas, QByteArray(), this);

        initEnableStatus();

        connect(mMouseDisableBtn, &KSwitchButton::stateChanged, this, &TouchpadUI::mouseDisableSlot);

        connect(mPointerSpeedSlider, &QSlider::valueChanged, this, &TouchpadUI::pointerSpeedSlot);

        connect(mTypingDisableBtn, &KSwitchButton::stateChanged, this, &TouchpadUI::typingDisableSlot);

        connect(mClickBtn, &KSwitchButton::stateChanged, this, &TouchpadUI::clickSlot);

        connect(mScrollSlideBtn, &KSwitchButton::stateChanged, this, &TouchpadUI::scrollSlideSlot);

        connect(mScrollTypeComBox, QOverload<const QString &>::of(&QComboBox::currentIndexChanged),
                this, &TouchpadUI::scrolltypeSlot);

        gsettingConnectUi();
    }
}

void TouchpadUI::gsettingConnectUi()
{
    //命令行指针速度改变 key 值，界面做出相应的改变
    connect(mTouchpadGsetting, &QGSettings::changed, this, [=](const QString &key) {
       if (key == kPointerSpeedKey) {
           int val = mTouchpadGsetting->get(kPointerSpeedKey).toInt();
           if (val < 1) {   //异常数据保护 避免手动更改gsetting配置文件导致鼠标过快或者过慢
               mPointerSpeedSlider->blockSignals(true);
               mPointerSpeedSlider->setValue(11);
               mPointerSpeedSlider->blockSignals(false);
           } else if (val > 8) {
               mPointerSpeedSlider->blockSignals(true);
               mPointerSpeedSlider->setValue(8 *10);
               mPointerSpeedSlider->blockSignals(false);
           }
           else {
               mPointerSpeedSlider->blockSignals(true);
               mPointerSpeedSlider->setValue(val *10);
               mPointerSpeedSlider->blockSignals(false);
           }
       }
    });
}

void TouchpadUI::initEnableStatus()
{
    // 插入鼠标时候禁用触摸板
    mMouseDisableBtn->blockSignals(true);
    mMouseDisableBtn->setChecked(mTouchpadGsetting->get(kMouseDisableKey).toBool());
    mMouseDisableBtn->blockSignals(false);

    // 初始化光标速度
    mPointerSpeedSlider->blockSignals(true);
    mPointerSpeedSlider->setValue(mTouchpadGsetting->get(kPointerSpeedKey).toInt()* 10);
    mPointerSpeedSlider->blockSignals(false);

    // 初始化打字时禁用触摸板
    mTypingDisableBtn->blockSignals(true);
    mTypingDisableBtn->setChecked(mTouchpadGsetting->get(kTypingDisableKey).toBool());
    mTypingDisableBtn->blockSignals(false);

    // 初始化触摸板轻触点击
    mClickBtn->blockSignals(true);
    mClickBtn->setChecked(mTouchpadGsetting->get(kClickKey).toBool());
    mClickBtn->blockSignals(false);

    // 初始化滚动条跟随手指滑动
    mScrollSlideBtn->blockSignals(true);
    mScrollSlideBtn->setChecked(!mTouchpadGsetting->get(kScrollSlideKey).toBool());
    mScrollSlideBtn->blockSignals(false);

    //初始化滚动
    mScrollTypeComBox->blockSignals(true);
    mScrollTypeComBox->setCurrentIndex(mScrollTypeComBox->findData(_findKeyScrollingType()));
    mScrollTypeComBox->blockSignals(false);

    // 边界滚动默认水平边界有效
    if (QString::compare(V_EDGE_KEY, mScrollTypeComBox->currentData().toString()) == 0) {
        mTouchpadGsetting->set(H_EDGE_KEY, true);
        mTouchpadGsetting->set(H_FINGER_KEY, false);
    }
    if (QString::compare(V_FINGER_KEY, mScrollTypeComBox->currentData().toString()) == 0) {
        mTouchpadGsetting->set(H_EDGE_KEY, false);
        mTouchpadGsetting->set(H_FINGER_KEY, true);
    }
    if (QString::compare(N_SCROLLING, mScrollTypeComBox->currentData().toString()) == 0) {
        mTouchpadGsetting->set(H_EDGE_KEY, false);
        mTouchpadGsetting->set(H_FINGER_KEY, false);
    }
}

QString TouchpadUI::_findKeyScrollingType()
{
    if (mTouchpadGsetting->get(V_EDGE_KEY).toBool()) {
        return V_EDGE_KEY;
    }

    if (mTouchpadGsetting->get(V_FINGER_KEY).toBool()) {
        return V_FINGER_KEY;
    }

    return N_SCROLLING;
}

/* slot functions */
void TouchpadUI::mouseDisableSlot(bool status)
{
    mTouchpadGsetting->set(kMouseDisableKey, status);
}

void TouchpadUI::pointerSpeedSlot(int value)
{
    mTouchpadGsetting->set(kPointerSpeedKey,value/10);
}

void TouchpadUI::typingDisableSlot(bool status)
{
    mTouchpadGsetting->set(kTypingDisableKey, status);
}

void TouchpadUI::clickSlot(bool status)
{
    mTouchpadGsetting->set(kClickKey, status);
}

void TouchpadUI::scrollSlideSlot(bool status)
{
    mTouchpadGsetting->set(kScrollSlideKey, !status);
}

void TouchpadUI::scrolltypeSlot()
{
    //旧滚动类型设置为false，新滚动类型设置为true
    QString data = mScrollTypeComBox->currentData().toString();
    // 禁用滚动
    if (QString::compare(data, N_SCROLLING) == 0) {
        mTouchpadGsetting->set(V_EDGE_KEY, false);
        mTouchpadGsetting->set(H_EDGE_KEY, false);
        mTouchpadGsetting->set(V_FINGER_KEY, false);
        mTouchpadGsetting->set(H_FINGER_KEY, false);
    }
    // 边界滚动:垂直边界+水平边界
    if (QString::compare(data, V_EDGE_KEY) == 0) {
        mTouchpadGsetting->set(V_EDGE_KEY, true);
        mTouchpadGsetting->set(H_EDGE_KEY, true);
        mTouchpadGsetting->set(V_FINGER_KEY, false);
        mTouchpadGsetting->set(H_FINGER_KEY, false);
    }
    // 中间双指:垂直中间+水平中间
    if (QString::compare(data, V_FINGER_KEY) == 0) {
        mTouchpadGsetting->set(V_EDGE_KEY, false);
        mTouchpadGsetting->set(H_EDGE_KEY, false);
        mTouchpadGsetting->set(V_FINGER_KEY, true);
        mTouchpadGsetting->set(H_FINGER_KEY, true);
    }
}

void TouchpadUI::playGif(ScreenGestureType type)
{
    switch (type) {
    case ScreenGestureType::threeFingerClick:
        m_movie->setFileName(":/gif/resources/"+projectCodeName+"三指轻点.gif");
        break;
    case ScreenGestureType::threeFingerSwipeDown:
        m_movie->setFileName(":/gif/resources/"+projectCodeName+"三指向下滑.gif");
        break;
    case ScreenGestureType::threeFingerSwipeUp:
        m_movie->setFileName(":/gif/resources/"+projectCodeName+"三指向上滑.gif");
        break;
    case ScreenGestureType::threeFingersHorizontalSwipe:
        m_movie->setFileName(":/gif/resources/"+projectCodeName+"三指左右滑动.gif");
        break;
    case ScreenGestureType::fourFingerClick:
        m_movie->setFileName(":/gif/resources/"+projectCodeName+"四指轻点.gif");
        break;
    case ScreenGestureType::fourFingersHorizontalSwipe:
        m_movie->setFileName(":/gif/resources/"+projectCodeName+"四指左右滑动.gif");
        break;
    default:
        break;
    }
    m_movie->stop();
    m_cycleNum = 0;
    m_movie->start();
}

void TouchpadUI::stopGif()
{
    m_movie->stop();
//     停止播放后，跳转到第一页
    m_movie->jumpToFrame(0);
}

void TouchpadUI::monitorIconThemeChange()
{
    QByteArray themeId(themeSchemas);
    if (QGSettings::isSchemaInstalled(themeId)) {
        m_iconThemeGSettings = new QGSettings(themeId);
        connect(m_iconThemeGSettings, &QGSettings::changed, this, [=](const QString &key) {
            if (key == "iconThemeName") {
                emit iconThemeChangedSignal();
            }
        });
    }
}
