#include "touchscreen-settings.h"
#include "touchscreen.h"
#include "gestureguidance.h"
#include "devicemonitoreventfilter.h"

#include <ukuisdk/kylin-com4cxx.h>

#include <string>

#include <QLocale>
#include <QTranslator>
#include <QApplication>
#include <QTouchDevice>
#include <QString>
#include <QWidget>
#include <QIcon>
#include <QDebug>
#include <QGSettings>
#include <QByteArray>

static const std::string PROJECT_V10SP1 = "V10SP1";
static const std::string PROJECT_INTEL  = "V10SP1-edu";
static const std::string PLUGINS_SCHEMA = "org.ukui.control-center.plugins";
static const std::string PLUGINS_DIR = "/org/ukui/control-center/plugins/";
static const std::string KEY_SHOW = "show";

TouchscreenSettings::TouchscreenSettings() : mFirstLoad(true)
{
    QTranslator* translator = new QTranslator(this);
    translator->load("/usr/share/touchscreen-settings/translations/" + QLocale::system().name());
    QApplication::installTranslator(translator);

    prjCodeName = QString::fromStdString(KDKGetPrjCodeName());
    productFeatures = QString::fromStdString(KDKGetOSRelease("PRODUCT_FEATURES"));

    m_deviceMonitor = new DeviceMonitorEventFilter(this);
    qApp->installNativeEventFilter(m_deviceMonitor);
    connect(m_deviceMonitor, &DeviceMonitorEventFilter::touchScreenChanged,
            this, &TouchscreenSettings::setShow);
}

TouchscreenSettings::~TouchscreenSettings()
{
    //! ukui-control-center is responsible for destroying pluginWidget
}

QString TouchscreenSettings::plugini18nName()
{
    QString pluginName;
    //! \note is same UI in mavis and laika
    if (prjCodeName == QString("PROJECT_INTEL"))
        pluginName = tr("GestureGuidance");
    else
        pluginName = tr("TouchScreen");

    if (productFeatures.toInt() == 3)
        pluginName = tr("TabletAndTouchScreen");

    return pluginName;
}

int TouchscreenSettings::pluginTypes()
{
    return FunType::DEVICES;
}

QWidget *TouchscreenSettings::pluginUi()
{
    qDebug() << "The current project is: " << prjCodeName;

    if (!mFirstLoad)
        return pluginWidget;

    if (prjCodeName == QString("PROJECT_INTEL")) {
        pluginWidget = new GestureGuidance;
    }
    else {
        pluginWidget = new TouchScreen;
    }

    mFirstLoad = false;
    pluginWidget->adjustSize();
    pluginWidget->setAttribute(Qt::WA_DeleteOnClose);


//    if (mFirstLoad) {
//        mFirstLoad = false;
//        pluginWidget = new TouchScreen;
//        pluginWidget->adjustSize();
//        pluginWidget->setAttribute(Qt::WA_DeleteOnClose);
//    }
    return pluginWidget;
}

bool TouchscreenSettings::isEnable() const
{
    QList<const QTouchDevice *> devices = QTouchDevice::devices();
    for (auto device: devices) {
        if(device->type() == QTouchDevice::TouchScreen)
            return true;
    }
    return false;
}

const QString TouchscreenSettings::name() const
{
    return QStringLiteral("TouchScreen");
}

bool TouchscreenSettings::isShowOnHomePage() const
{
    return false;
}

QIcon TouchscreenSettings::icon() const
{
    return QIcon::fromTheme("ukui-gesture-symbolic");
}

QString TouchscreenSettings::translationPath() const
{
    return "/usr/share/touchscreen-settings/translations/%1.ts";
}

void TouchscreenSettings::initSearchText()
{
}

void TouchscreenSettings::setShow(bool isShow)
{
    const QByteArray id(PLUGINS_SCHEMA.c_str());
    if (!QGSettings::isSchemaInstalled(id)) {
        qWarning() << __FILE__ << __FUNCTION__ << __LINE__
                   << "schema :" << id << "is not install";
        return;
    }

    QString path(QString::fromStdString(PLUGINS_DIR + "TouchScreen/"));
    QGSettings settings(id, path.toUtf8());

    // It is signaled whenever the device list changes,
    // so it is possible that the touchscreen state has not changed
    if (settings.get(KEY_SHOW.c_str()).toBool() == isShow)
        return;

    settings.set(KEY_SHOW.c_str(), isShow);
    qDebug() << __FILE__ << __FUNCTION__ << __LINE__
             << "Set GSetting" << id << path << KEY_SHOW.c_str() << isShow;
}
