#ifndef TOUCHSCREEN_H
#define TOUCHSCREEN_H

#include <QWidget>

namespace Ui {
class TouchScreen;
}

namespace kdk {
class KSwitchButton;
}

enum class ScreenGestureType
{
    bottomEdgeSwipe,
    topEdgeSwipe,
    rightEdgeSwipe,
    fourFingersSwipeDown,
    fourFingersHorizontalSwipe,
    unknown
};

class QMovie;
class QGSettings;
class GestureWidget;
class TouchScreen : public QWidget
{
    Q_OBJECT
public:
    explicit TouchScreen(QWidget *parent = nullptr);
    ~TouchScreen();

    bool eventFilter(QObject *watched, QEvent *event);
private:
    ///
    /// \brief 监听图标主题的变化，图标主题改变后修改播放按钮的图标
    ///
    void monitorIconThemeChange();

public slots:
    void stopGif();
signals:
    void iconThemeChangedSignal();

private slots:
    void setAutoSwitchTabletBtn(bool checked);
private:
    void initUI();
    void loadGif();
    void playGif(ScreenGestureType type);
    bool autoSwitchTablet();
    void setAutoSwitchTablet(bool state);

    Ui::TouchScreen *ui;
    quint32 m_cycleNum;
    QMovie *m_movie;
    QVector<GestureWidget *> m_gestureArray;
    QGSettings *m_iconThemeGSettings;
    kdk::KSwitchButton* m_autoSwitchTablet;
    int m_productFeat;
};

#endif // TOUCHSCREEN_H
